package application;

import java.io.File;
import java.util.HashSet;

import javafx.application.Application;
import javafx.beans.binding.DoubleBinding;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.scene.Node;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.ScrollPane;
import javafx.scene.control.Separator;
import javafx.scene.control.ToggleButton;
import javafx.scene.control.ToolBar;
import javafx.scene.layout.BorderPane;
import javafx.scene.paint.Color;
import javafx.stage.FileChooser;
import javafx.stage.FileChooser.ExtensionFilter;
import javafx.stage.Stage;
import javafx.util.converter.IntegerStringConverter;
import jfxtras.labs.internal.scene.control.skin.ListSpinnerCaspianSkin;
import jfxtras.labs.internal.scene.control.skin.ListSpinnerCaspianSkin.ArrowDirection;
import jfxtras.labs.internal.scene.control.skin.ListSpinnerCaspianSkin.ArrowPosition;
import jfxtras.labs.scene.control.ListSpinner;
import level.LevelCanvas;
import level.blocks.Block;

/**
 * The Teeter Level Builder Main Class.
 */
public class Main extends Application {

	/** The root border pane. */
	private static BorderPane root = new BorderPane();

	/** The level canvas. */
	private static LevelCanvas levelCanvas = new LevelCanvas(16, 16);

	/** The file chooser. */
	private static FileChooser fileChooser = new FileChooser();

	/** The pen button. */
	private static ToggleButton penButton = new ToggleButton("pen");

	/** The stage. */
	private static Stage stage;

	/**
	 * Gets the stage.
	 * 
	 * @return the stage
	 */
	public static Stage getStage() {
		return stage;
	}

	/**
	 * Gets the level canvas.
	 * 
	 * @return the level canvas
	 */
	public static LevelCanvas getLevelCanvas() {
		return levelCanvas;
	}

	@Override
	public void start(final Stage primaryStage) {
		fileChooser.getExtensionFilters().add(
				new ExtensionFilter("Teeter Level Image (*.png)", "*.png"));
		stage = primaryStage;
		stage.setTitle("Teeter Level Builder");
		try {
			Scene scene = new Scene(root, 1000, 650);
			primaryStage.setScene(scene);
			primaryStage.show();
			scene.getStylesheets().add(
					getClass().getResource("application.css").toExternalForm());

			root.setTop(buildTopMenu());
			root.setCenter(buildLevelCanvas());

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private static File file;

	private static Node buildTopMenu() {
		final Label fileLabel = new Label("no file selected");

		final ListSpinner<Integer> widthSpinner = new ListSpinner<>(1, 30);
		ListSpinnerCaspianSkin<Integer> widthSpinnerSkin = new ListSpinnerCaspianSkin<>(
				widthSpinner);
		widthSpinnerSkin.setArrowDirection(ArrowDirection.HORIZONTAL);
		widthSpinnerSkin.setArrowPosition(ArrowPosition.SPLIT);
		widthSpinnerSkin.setMaxWidth(100);
		widthSpinner.setSkin(widthSpinnerSkin);
		widthSpinner.setEditable(true);
		widthSpinner.setStringConverter(new IntegerStringConverter());
		widthSpinner.setValue(16);
		widthSpinner.valueProperty().addListener(new ChangeListener<Integer>() {
			@Override
			public void changed(
					final ObservableValue<? extends Integer> pObservable,
					final Integer pOldValue, final Integer pNewValue) {
				levelCanvas.setColumns(pNewValue);
			}
		});

		final ListSpinner<Integer> heightSpinner = new ListSpinner<>(1, 30);
		ListSpinnerCaspianSkin<Integer> heightSpinnerSkin = new ListSpinnerCaspianSkin<>(
				heightSpinner);
		heightSpinnerSkin.setArrowDirection(ArrowDirection.HORIZONTAL);
		heightSpinnerSkin.setArrowPosition(ArrowPosition.SPLIT);
		heightSpinnerSkin.setMaxWidth(100);
		heightSpinner.setSkin(heightSpinnerSkin);
		heightSpinner.setEditable(true);
		heightSpinner.setStringConverter(new IntegerStringConverter());
		heightSpinner.setValue(16);
		heightSpinner.valueProperty().addListener(
				new ChangeListener<Integer>() {
					@Override
					public void changed(
							final ObservableValue<? extends Integer> pObservable,
							final Integer pOldValue, final Integer pNewValue) {
						levelCanvas.setRows(pNewValue);
					}
				});

		Button newButton = new Button("new");
		newButton.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(final ActionEvent pEvent) {
				file = null;
				fileLabel.setText("no file selected");
				levelCanvas.renew(widthSpinner.getValue(),
						heightSpinner.getValue());
			}
		});

		Button loadButton = new Button("load");
		loadButton.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(final ActionEvent pEvent) {
				if (file != null && file.getParentFile() != null) {
					fileChooser.setInitialDirectory(file.getParentFile());
				}
				file = fileChooser.showOpenDialog(stage);
				if (file != null) {
					fileLabel.setText(file.getName());
					levelCanvas.loadImage(file);
					widthSpinner.setValue(levelCanvas.getColumns());
					heightSpinner.setValue(levelCanvas.getRows());
				}
			}
		});

		Button saveButton = new Button("save");
		saveButton.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(final ActionEvent pEvent) {
				if (file == null) {
					file = fileChooser.showSaveDialog(stage);
				}
				if (file != null) {
					fileLabel.setText(file.getName());
					levelCanvas.saveImage(file);
				}
			}
		});

		Button saveAsButton = new Button("save as");
		saveAsButton.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(final ActionEvent pEvent) {
				if (file != null && file.getParentFile() != null) {
					fileChooser.setInitialDirectory(file.getParentFile());
				}
				File f = fileChooser.showSaveDialog(stage);
				if (f != null) {
					file = f;
					fileLabel.setText(file.getName());
					levelCanvas.saveImage(file);
				}
			}
		});

		return new ToolBar(new Label("Width:"), widthSpinner, new Label(
				"Height:"), heightSpinner, newButton, new Separator(),
				loadButton, saveButton, saveAsButton, fileLabel,
				new Separator(), penButton);
	}

	private static Node buildLevelCanvas() {
		final ScrollPane sp = new ScrollPane();
		// to center the LevelCanvas, we have to scale it to the viewport
		// size of the ScrollPane
		levelCanvas.minHeightProperty().bind(new DoubleBinding() {
			{
				super.bind(sp.viewportBoundsProperty());
			}

			@Override
			protected double computeValue() {
				if (sp.viewportBoundsProperty().get() != null) {
					return sp.viewportBoundsProperty().get().getHeight();
				}
				return sp.getHeight();
			}
		});
		levelCanvas.minWidthProperty().bind(new DoubleBinding() {
			{
				super.bind(sp.viewportBoundsProperty());
			}

			@Override
			protected double computeValue() {
				if (sp.viewportBoundsProperty().get() != null) {
					return sp.viewportBoundsProperty().get().getWidth();
				}
				return sp.getWidth();
			}
		});
		sp.setContent(levelCanvas);
		return sp;
	}

	/** The active block. */
	private static Block activeBlock;

	/**
	 * Show the right menu for a block.
	 * 
	 * @param block
	 *            the block to show the menu for
	 */
	public static void showRightMenuFor(final Block block) {
		if (activeBlock != null) {
			activeBlock.setFocus(false);
		}
		activeBlock = block;
		if (activeBlock == null) {
			root.setRight(null);
		} else {
			root.setRight(activeBlock.getMenu());
			activeBlock.setFocus(true);
		}
	}

	private static Block lastClicked;

	/**
	 * Handle a mouse click on a block.
	 * 
	 * @param pBlock
	 *            the block that was clicked on
	 * @param override
	 *            true to override the old block with the active one
	 */
	public static void handleBlockClicked(final Block pBlock,
			final boolean override) {
		Block newActive = pBlock;
		if (penButton.isSelected() || override) {
			newActive = levelCanvas.setBlockType(pBlock, activeBlock, true);
		}
		showRightMenuFor(newActive);
	}

	/**
	 * Handle a mouse over on a block.
	 * 
	 * @param pBlock
	 *            the block that was clicked on
	 */
	public static void handleBlockOver(final Block pBlock) {
		if (lastClicked != null && lastClicked.getX() == pBlock.getX()
				&& lastClicked.getY() == pBlock.getY()) {
			return;
		}
		if (penButton.isSelected()) {
			levelCanvas.setBlockType(pBlock, activeBlock, true);
			lastClicked = pBlock;
		}
	}

	/**
	 * The main method.
	 * 
	 * @param args
	 *            the arguments
	 */
	public static void main(final String[] args) {
		// testEncoding();
		launch(args);
	}

	/**
	 * Test the encoding.
	 */
	@SuppressWarnings("unused")
	private static void testEncoding() {
		HashSet<Integer> brightnessFails = new HashSet<>();

		int[] hues = { 0, 45, 90, 135, 180, 225, 270, 315 };
		int hits = 0;
		for (int hue : hues) {
			System.out.println("\nHue: " + hue);
			int huehits = 0;
			for (int saturation = 50; saturation <= 100; saturation += 3) {
				int saturationhits = 0;
				for (int brightness = 0; brightness <= 100; brightness++) {
					Color color = Color.hsb(hue, saturation / 100.0,
							brightness / 100.0);
					int r = (int) (color.getRed() * 255);
					int g = (int) (color.getGreen() * 255);
					int b = (int) (color.getBlue() * 255);

					Color color2 = Color.rgb(r, g, b);
					int hue2 = (int) (color2.getHue() + 0.5);
					int brightness2 = (int) (color2.getBrightness() * 100 + 0.5);
					int saturation2 = (int) (color2.getSaturation() * 100 + 0.5);

					if (equal(hue, hue2, 10)
							&& equal(brightness, brightness2, 0)
							&& equal(saturation, saturation2, 1)) {
						System.out.println("hsb: " + hue + ", " + saturation
								+ ", " + brightness + " - " + hue2 + ", "
								+ saturation2 + ", " + brightness2);
						hits++;
						huehits++;
						saturationhits++;
					} else {
						brightnessFails.add(brightness);
					}
				}
				System.out.println(saturationhits + " Saturation Hits");
			}
			System.out.println(huehits + " Hue Hits");
		}
		System.out.println(hits + " Hits");

		int bhits = 0;
		for (int b = 0; b <= 100; b++) {
			if (!brightnessFails.contains(b)) {
				System.out.println("Hit: " + b);
				bhits++;
			} else {
				System.out.println("Fail: " + b);
			}
		}
		System.out.println("Hits: " + bhits);
	}

	private static boolean equal(final int number1, final int number2,
			final int delta) {
		return Math.abs(number1 - number2) <= delta;
	}
}
