package level;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;

import javafx.embed.swing.SwingFXUtils;
import javafx.geometry.Pos;
import javafx.scene.Node;
import javafx.scene.image.Image;
import javafx.scene.image.PixelReader;
import javafx.scene.image.PixelWriter;
import javafx.scene.image.WritableImage;
import javafx.scene.layout.GridPane;

import javax.imageio.ImageIO;

import level.blocks.Block;
import level.blocks.BlockBuilder;
import application.Main;

/**
 * Draws a whole level image on a grid for editing.
 */
public class LevelCanvas extends GridPane {

	private WritableImage levelImage;

	/**
	 * Load image from a file.
	 * 
	 * @param file
	 *            the file name
	 */
	public void loadImage(final File file) {
		if (file.exists()) {
			try {
				Image image = new Image(file.toURI().toString());
				setImage(image);
			} catch (IllegalArgumentException iae) {
				return;
			}
		}
	}

	/**
	 * Save the image to a file.
	 * 
	 * @param file
	 *            the file
	 */
	public void saveImage(final File file) {
		try {
			ImageIO.write(SwingFXUtils.fromFXImage(getImage(), null), "png",
					file);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Sets the image.
	 * 
	 * @param pLevelImage
	 *            the new image
	 */
	public void setImage(final Image pLevelImage) {
		setImage(new WritableImage(pLevelImage.getPixelReader(),
				(int) pLevelImage.getWidth(), (int) pLevelImage.getHeight()));
	}

	/**
	 * Sets the image.
	 * 
	 * @param pLevelImage
	 *            the new image
	 */
	private void setImage(final WritableImage pLevelImage) {
		levelImage = pLevelImage;
		getChildren().clear();

		PixelReader pr = levelImage.getPixelReader();

		Block first = null;

		for (int y = 0; y < pLevelImage.getHeight(); ++y) {
			for (int x = 0; x < pLevelImage.getWidth(); ++x) {
				Block block = BlockBuilder.build(x, y, pr.getColor(x, y));
				add(block, x, y);
				if (y == 0 && x == 0) {
					first = block;
				}
			}
		}
		Main.showRightMenuFor(first);
	}

	/**
	 * Clear the canvas and create a new one with the specified dimensions.
	 * 
	 * @param pColumns
	 *            the number of columns
	 * @param pRows
	 *            the number of rows
	 */
	public void renew(final int pColumns, final int pRows) {
		setImage(new WritableImage(pColumns, pRows));
	}

	/**
	 * Sets the number of columns.
	 * 
	 * @param pColumns
	 *            the new number of columns
	 */
	public void setColumns(final int pColumns) {
		if (pColumns != (int) levelImage.getWidth()) {
			WritableImage newImage = new WritableImage(pColumns,
					(int) levelImage.getHeight());
			fitToImage(newImage);
		}
	}

	/**
	 * Gets the number of columns.
	 * 
	 * @return the number of columns
	 */
	public int getColumns() {
		return (int) levelImage.getWidth();
	}

	/**
	 * Sets the number of rows.
	 * 
	 * @param pRows
	 *            the new number of rows
	 */
	public void setRows(final int pRows) {
		if (pRows != (int) levelImage.getHeight()) {
			WritableImage newImage = new WritableImage(
					(int) levelImage.getWidth(), pRows);
			fitToImage(newImage);
		}
	}

	/**
	 * Gets the number of rows.
	 * 
	 * @return the number of rows
	 */
	public int getRows() {
		return (int) levelImage.getHeight();
	}

	/**
	 * Copy the contents of the old image to the new one.<br>
	 * If the new image is bigger, pixels will be left blank
	 * 
	 * @param target
	 *            the target image
	 */
	private void fitToImage(final WritableImage target) {
		PixelWriter pw = target.getPixelWriter();
		Iterator<Node> i = getChildren().iterator();
		while (i.hasNext()) {
			Node n = i.next();
			if (n instanceof Block) {
				Block b = (Block) n;
				if (b.getX() < target.getWidth()
						&& b.getY() < target.getHeight()) {
					pw.setColor(b.getX(), b.getY(), b.getColor());
				} else {
					i.remove();
				}
			}
		}
		setImage(target);
	}

	/**
	 * Gets the image.<br>
	 * Internally this is when the levelImage data is updated according to the
	 * block data.
	 * 
	 * @return the image
	 */
	public Image getImage() {
		PixelWriter pw = levelImage.getPixelWriter();
		for (Node n : getChildren()) {
			if (n instanceof Block) {
				Block b = (Block) n;
				pw.setColor(b.getX(), b.getY(), b.getColor());
			}
		}
		return levelImage;
	}

	/**
	 * Instantiates a new level canvas.
	 * 
	 * @param pColumns
	 *            the number of columns
	 * @param pRows
	 *            the number of rows
	 */
	public LevelCanvas(final int pColumns, final int pRows) {
		super();

		setAlignment(Pos.CENTER);
		setHgap(2);
		setVgap(2);

		renew(pColumns, pRows);
	}

	/**
	 * Sets the block type of a level cell.<br>
	 * If the old block is of the same type as the prototype, this will do
	 * nothing.
	 * 
	 * @param pOldBlock
	 *            the old block on the position to be updated
	 * @param pSelectedPrototype
	 *            the selected prototype to use for the cell
	 * @param override
	 *            true to override all parameters
	 * @return the resulting block
	 */
	public Block setBlockType(final Block pOldBlock,
			final Block pSelectedPrototype, final boolean override) {
		if (pSelectedPrototype.getClass() == pOldBlock.getClass() && !override) {
			return pOldBlock;
		}

		getChildren().remove(pOldBlock);
		Block clone = pSelectedPrototype.clone();
		clone.setX(pOldBlock.getX());
		clone.setY(pOldBlock.getY());
		add(clone, pOldBlock.getX(), pOldBlock.getY());
		return clone;
	}
}
