package level.blocks;

import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.scene.Node;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.control.ComboBox;
import javafx.scene.control.ListCell;
import javafx.scene.control.ListView;
import javafx.scene.control.TitledPane;
import javafx.scene.input.ClipboardContent;
import javafx.scene.input.DragEvent;
import javafx.scene.input.Dragboard;
import javafx.scene.input.MouseEvent;
import javafx.scene.input.TransferMode;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.util.Callback;
import application.Main;

/**
 * A block of the level.
 */
public class Block extends Canvas implements Cloneable, ChangeListener<Object> {

	/** The x coordinate. */
	private int x;

	/**
	 * Gets the x.
	 * 
	 * @return the x
	 */
	public int getX() {
		return x;
	}

	/**
	 * Sets the x.
	 * 
	 * @param pX
	 *            the new x
	 */
	public void setX(final int pX) {
		x = pX;
	}

	/** The y coordinate. */
	private int y;

	/**
	 * Gets the y.
	 * 
	 * @return the y
	 */
	public int getY() {
		return y;
	}

	/**
	 * Sets the y.
	 * 
	 * @param pY
	 *            the new y
	 */
	public void setY(final int pY) {
		y = pY;
	}

	/**
	 * Instantiates a new block.
	 * 
	 * @param pX
	 *            the x coordinate
	 * @param pY
	 *            the y coordinate
	 */
	public Block(final int pX, final int pY) {
		super(30, 30);
		x = pX;
		y = pY;
		init();

		setOnMouseClicked(new EventHandler<MouseEvent>() {
			@Override
			public void handle(final MouseEvent pEvent) {
				Main.handleBlockClicked(Block.this, false);
			}
		});

		setOnDragDetected(new EventHandler<MouseEvent>() {
			@Override
			public void handle(final MouseEvent pEvent) {
				Dragboard db = startDragAndDrop(TransferMode.COPY);
				Main.handleBlockClicked(Block.this, false);
				ClipboardContent content = new ClipboardContent();
				content.putString("");
				db.setContent(content);
				pEvent.consume();
			}
		});

		setOnDragOver(new EventHandler<DragEvent>() {
			@Override
			public void handle(final DragEvent pEvent) {
				pEvent.acceptTransferModes(TransferMode.COPY);
				Main.handleBlockOver(Block.this);
				pEvent.consume();
			}
		});

		setOnDragDropped(new EventHandler<DragEvent>() {
			@Override
			public void handle(final DragEvent pEvent) {
				pEvent.acceptTransferModes(TransferMode.COPY);
				Main.handleBlockClicked(Block.this, true);
				pEvent.consume();
			}
		});

		draw();
	}

	/**
	 * Initializes the fields of the object inside the super constructor.
	 */
	protected void init() {
		// nothing to do here
	}

	/**
	 * Gets the color of the block.
	 * 
	 * @return the color encoding of the block
	 */
	public final Color getColor() {
		return Color.hsb(getHue(), getSaturation(), getBrightness(),
				getTransparency());
	}

	/**
	 * Checks if this is the hue of the block type.<br>
	 * The test should compare the hue within a limited environment to fix
	 * rounding problems.
	 * 
	 * @param hue
	 *            the hue to check
	 * @return true, if it is the hue of the block type.
	 */
	public boolean isHue(final int hue) {
		return false;
	}

	/**
	 * Gets the hue value for the type of the block.
	 * 
	 * @return the hue value between 0 and 360
	 */
	protected int getHue() {
		return 0;
	}

	/**
	 * Gets the saturation value for the type specifics of the block.
	 * 
	 * @return the saturation value between 0.0 and 1.0
	 */
	protected double getSaturation() {
		return 0.0;
	}

	/**
	 * Sets the saturation value for the type specifics of the block.
	 * 
	 * @param pSaturation
	 *            the new saturation value between 0.0 and 1.0
	 */
	public void setSaturation(final double pSaturation) {
		return;
	}

	/**
	 * Gets the brightness value for the shape of the block.
	 * 
	 * @return the brightness value between 0.0 and 1.0
	 */
	protected double getBrightness() {
		return 0.0;
	}

	/**
	 * Sets the brightness value for the shape of the block.
	 * 
	 * @param pBrightness
	 *            the new brightness value between 0.0 and 1.0
	 */
	public void setBrightness(final double pBrightness) {
		return;
	}

	/**
	 * Gets the transparency value for the id of the block.
	 * 
	 * @return the transparency value between 0.0 and 1.0
	 */
	protected double getTransparency() {
		return 1.0;
	}

	/**
	 * Sets the transparency value for the id of the block.
	 * 
	 * @param pTransparency
	 *            the new transparency value between 0.0 and 1.0
	 */
	public void setTransparency(final double pTransparency) {
		return;
	}

	@Override
	public void changed(final ObservableValue<? extends Object> pObservable,
			final Object pOldValue, final Object pNewValue) {
		draw();
	}

	/**
	 * Draw the block on the canvas.
	 */
	public void draw() {
		GraphicsContext gc = getGraphicsContext2D();
		gc.setFill(Color.WHITE);
		gc.fillRect(0, 0, getWidth(), getHeight());
		Color c = getColor();
		if (c.getBrightness() > 0) {
			gc.setFill(getColor());
			gc.fillRect(0, 0, getWidth(), getHeight());
		}
		if (focused) {
			gc.setStroke(Color.BLACK);
			gc.setLineWidth(3);
			gc.strokeRect(1, 1, getWidth() - 2, getHeight() - 2);

			gc.setStroke(Color.WHITE);
			gc.setLineWidth(2);
			gc.strokeRect(3, 3, getWidth() - 6, getHeight() - 6);
		}
	}

	/**
	 * Returns an instance of the block type with cloned coordinates.<br>
	 * Is called during {@link Block#clone()} to build the correct block type.
	 * Saturation, Brightness and Transparency will be cloned in
	 * {@link Block#clone()}.
	 * 
	 * @return a basic block of this block's type with cloned coordinates
	 */
	protected Block subClone() {
		return new Block(getX(), getY());
	}

	@Override
	public final Block clone() {
		Block clone = subClone();
		clone.setSaturation(getSaturation());
		clone.setBrightness(getBrightness());
		clone.setTransparency(getTransparency());
		return clone;
	}

	@Override
	public String toString() {
		return "Empty Block at (" + getX() + "|" + getY() + ")";
	}

	/**
	 * Gets the block's type name.
	 * 
	 * @return the type name of the block
	 */
	public String getTypeString() {
		return "Empty (Hole)";
	}

	/** The menu. */
	private Node menu;

	/**
	 * Gets the hue menu for type selection.
	 * 
	 * @return the menu to select a block type
	 */
	protected final Node getHueMenu() {
		TitledPane hueMenu = new TitledPane();
		hueMenu.setCollapsible(false);
		hueMenu.setText("Type selection");

		GridPane controls = new GridPane();
		controls.getStyleClass().add("grid-pane");

		final ComboBox<Block> prototypeCombo = new ComboBox<>(
				BlockBuilder.getPrototypes());
		for (Block b : prototypeCombo.getItems()) {
			if (b.getClass() == this.getClass()) {
				prototypeCombo.getSelectionModel().select(b);
				break;
			}
		}
		prototypeCombo.setButtonCell(new BlockTypeListCell());
		prototypeCombo
				.setCellFactory(new Callback<ListView<Block>, ListCell<Block>>() {
					@Override
					public ListCell<Block> call(final ListView<Block> pParam) {
						return new BlockTypeListCell();
					}
				});
		prototypeCombo.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(final ActionEvent pEvent) {
				Block result = Main.getLevelCanvas().setBlockType(Block.this,
						prototypeCombo.getValue(), false);
				Main.showRightMenuFor(result);
			}
		});

		controls.add(prototypeCombo, 0, 0);

		hueMenu.setContent(controls);

		return hueMenu;
	}

	/**
	 * Gets the saturation menu for type specifics.
	 * 
	 * @return the sub menu for the block type
	 */
	protected Node getSaturationMenu() {
		return null;
	}

	/**
	 * Gets the brightness menu for block shape.
	 * 
	 * @return the menu to configure the block's shape
	 */
	protected Node getBrightnessMenu() {
		return null;
	}

	/**
	 * Gets the transparency menu for block identifiers.
	 * 
	 * @return the menu to set the block's id.
	 */
	protected Node getTransparencyMenu() {
		return null;
	}

	/**
	 * Gets the menu for this block.
	 * 
	 * @return the menu
	 */
	public final Node getMenu() {
		if (menu == null) {
			VBox m = new VBox(5);

			Node hue = getHueMenu();
			if (hue != null) {
				m.getChildren().add(hue);
			}

			Node brightness = getBrightnessMenu();
			if (brightness != null) {
				m.getChildren().add(brightness);
			}

			Node saturation = getSaturationMenu();
			if (saturation != null) {
				m.getChildren().add(saturation);
			}

			Node transparency = getTransparencyMenu();
			if (transparency != null) {
				m.getChildren().add(transparency);
			}

			menu = m;
		}

		return menu;
	}

	/** The focused state. */
	private boolean focused;

	/**
	 * Sets the focus.
	 * 
	 * @param pFocused
	 *            the new focus
	 */
	public void setFocus(final boolean pFocused) {
		focused = pFocused;
		draw();
	}
}
