package level.blocks;

import javafx.beans.property.SimpleIntegerProperty;
import javafx.collections.FXCollections;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.scene.Node;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.control.TitledPane;
import javafx.scene.layout.GridPane;

/**
 * Trigger the action of all blocks with the same ID.
 */
public class ButtonBlock extends StandardBlock {

	/**
	 * The different types of Buttons
	 */
	public enum ButtonKind {
		/** Once activated, a trigger will become non-reactive. */
		Trigger,
		/** A push-button snaps back in its initial state after a short time. */
		PushButton,
		/** A switch toggles it state. */
		Switch;
	}

	/** The kind of the button. */
	private ButtonKind kind;

	private static OffsetValueList<Integer> pushDurationOffsets = new OffsetValueList<>(
			new OffsetValue<>(0, 1), new OffsetValue<>(3, 2),
			new OffsetValue<>(6, 3), new OffsetValue<>(9, 4),
			new OffsetValue<>(12, 5), new OffsetValue<>(15, 6),
			new OffsetValue<>(18, 7), new OffsetValue<>(21, 8),
			new OffsetValue<>(24, 9), new OffsetValue<>(27, 10));

	private SimpleIntegerProperty pushDuration;

	/**
	 * Instantiates a new Button block.
	 * 
	 * @param pX
	 *            the x coordinate
	 * @param pY
	 *            the y coordinate
	 */
	public ButtonBlock(final int pX, final int pY) {
		super(pX, pY);
	}

	@Override
	protected void init() {
		super.init();
		kind = ButtonKind.Trigger;
		pushDuration = new SimpleIntegerProperty(4);
		pushDuration.addListener(this);
	}

	@Override
	protected int getHue() {
		return 270;
	}

	@Override
	protected double getSaturation() {
		switch (kind) {
		default:
		case Trigger:
			return 1.0;
		case PushButton:
			return 0.60 + pushDurationOffsets.getByValue(pushDuration.get())
					.getOffset() / 100.0;
		case Switch:
			return 0.50;
		}
	}

	@Override
	public void setSaturation(final double pSaturation) {
		int saturation = (int) (pSaturation * 100.0 + 0.5);
		pushDuration.set(4);
		if (saturation >= 91) { // "== 100"
			kind = ButtonKind.Trigger;
		} else if (saturation >= 59) { // ">= 60"
			kind = ButtonKind.PushButton;
			OffsetValue<Integer> i = pushDurationOffsets
					.getByOffset(saturation - 60);
			if (i != null) {
				pushDuration.set(i.getValue());
			}
		} else if (saturation >= 49) { // "== 50"
			kind = ButtonKind.Switch;
		} else { // fallback for false encoding
			kind = ButtonKind.Trigger;
		}
		draw();
	}

	@Override
	protected boolean needsId() {
		return true;
	}

	@Override
	protected ButtonBlock subClone() {
		return new ButtonBlock(getX(), getY());
	}

	@Override
	public String toString() {
		return "Button Block [" + kind + "] at (" + getX() + "|" + getY() + ")";
	}

	@Override
	public String getTypeString() {
		return "Button";
	}

	@Override
	protected Node getSaturationMenu() {
		TitledPane saturationMenu = new TitledPane();
		saturationMenu.setCollapsible(false);
		saturationMenu.setText("Button type");

		GridPane controls = new GridPane();
		controls.getStyleClass().add("grid-pane");

		final ComboBox<ButtonKind> kindCombo = new ComboBox<>(
				FXCollections.observableArrayList(ButtonKind.values()));
		kindCombo.getSelectionModel().select(kind);
		kindCombo.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(final ActionEvent pEvent) {
				kind = kindCombo.getValue();
				changed(null, null, null);
			}
		});
		controls.add(kindCombo, 0, 0);

		Label pushDurationLabel = new Label("Duration:");
		pushDurationLabel.disableProperty().bind(
				kindCombo.valueProperty().isNotEqualTo(ButtonKind.PushButton));
		controls.add(pushDurationLabel, 0, 1);

		ComboBox<Integer> pushDurationCombo = new ComboBox<>(
				FXCollections
						.observableArrayList(1, 2, 3, 4, 5, 6, 7, 8, 9, 10));
		pushDurationCombo.getSelectionModel().select(pushDuration.get());
		pushDurationCombo.disableProperty().bind(
				kindCombo.valueProperty().isNotEqualTo(ButtonKind.PushButton));
		pushDuration.bind(pushDurationCombo.valueProperty());
		controls.add(pushDurationCombo, 1, 1);

		saturationMenu.setContent(controls);

		return saturationMenu;
	}
}
