package level.blocks;

import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleIntegerProperty;
import javafx.collections.FXCollections;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.scene.Node;
import javafx.scene.control.CheckBox;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.control.TitledPane;
import javafx.scene.layout.GridPane;
import jfxtras.labs.internal.scene.control.skin.ListSpinnerCaspianSkin;
import jfxtras.labs.internal.scene.control.skin.ListSpinnerCaspianSkin.ArrowDirection;
import jfxtras.labs.internal.scene.control.skin.ListSpinnerCaspianSkin.ArrowPosition;
import jfxtras.labs.scene.control.ListSpinner;

/**
 * Start, Finish and Checkpoints between them.
 */
public class CheckpointBlock extends StandardBlock {

	/**
	 * The different types of Checkpoints
	 */
	public enum CheckpointKind {
		/** Specifies the coordinate, on which the ball is spawned. */
		Start,
		/** A player should visit checkpoints first before finishing the level. */
		Checkpoint,
		/** A player has to reach an end block to finish the level. */
		End;
	}

	/** The kind of the checkpoint. */
	private CheckpointKind kind;

	/**
	 * The mandatory property.<br>
	 * Only used for {@link CheckpointKind#Checkpoint Checkpoints}
	 */
	private SimpleBooleanProperty mandatory;

	/**
	 * The points the player receives.<br>
	 * Only used for {@link CheckpointKind#Checkpoint Checkpoints} and
	 * {@link CheckpointKind#End End blocks}
	 */
	private SimpleIntegerProperty points;

	private static OffsetValueList<Integer> checkpointPointOffsets = new OffsetValueList<>(
			new OffsetValue<>(0, 10), new OffsetValue<>(3, 20),
			new OffsetValue<>(6, 30));

	private static OffsetValueList<Integer> endPointOffsets = new OffsetValueList<>(
			new OffsetValue<>(0, 50), new OffsetValue<>(3, 100),
			new OffsetValue<>(6, 150), new OffsetValue<>(9, 200),
			new OffsetValue<>(12, 250), new OffsetValue<>(15, 300),
			new OffsetValue<>(18, 350));

	/**
	 * Instantiates a new checkpoint block.
	 * 
	 * @param pX
	 *            the x coordinate
	 * @param pY
	 *            the y coordinate
	 */
	public CheckpointBlock(final int pX, final int pY) {
		super(pX, pY);
	}

	@Override
	protected void init() {
		super.init();
		kind = CheckpointKind.Checkpoint;
		mandatory = new SimpleBooleanProperty(false);
		mandatory.addListener(this);
		points = new SimpleIntegerProperty(10);
		points.addListener(this);
	}

	@Override
	protected int getHue() {
		return 135;
	}

	@Override
	protected double getSaturation() {
		switch (kind) {
		case Start:
			return 1.0;
		default:
		case Checkpoint:
			if (mandatory.get()) {
				return 0.80 + checkpointPointOffsets.getByValue(points.get())
						.getOffset() / 100.0;
			} else {
				return 0.70 + checkpointPointOffsets.getByValue(points.get())
						.getOffset() / 100.0;
			}
		case End:
			return 0.50 + endPointOffsets.getByValue(points.get()).getOffset() / 100.0;
		}
	}

	@Override
	public void setSaturation(final double pSaturation) {
		int saturation = (int) (pSaturation * 100.0 + 0.5);
		mandatory.set(false);
		points.set(10);
		if (saturation >= 91) { // "== 100"
			kind = CheckpointKind.Start;
		} else if (saturation >= 79) { // ">= 80"
			kind = CheckpointKind.Checkpoint;
			mandatory.set(true);
			OffsetValue<Integer> i = checkpointPointOffsets
					.getByOffset(saturation - 80);
			if (i != null) {
				points.set(i.getValue());
			} else {
				points.set(checkpointPointOffsets.getMinValue());
			}
		} else if (saturation >= 69) { // ">= 70"
			kind = CheckpointKind.Checkpoint;
			mandatory.set(false);
			OffsetValue<Integer> i = checkpointPointOffsets
					.getByOffset(saturation - 70);
			if (i != null) {
				points.set(i.getValue());
			} else {
				points.set(checkpointPointOffsets.getMinValue());
			}
		} else if (saturation >= 49) { // ">= 50"
			kind = CheckpointKind.End;
			OffsetValue<Integer> i = endPointOffsets
					.getByOffset(saturation - 50);
			if (i != null) {
				points.set(i.getValue());
			} else {
				points.set(endPointOffsets.getMinValue());
			}
		} else { // fallback for false encoding
			kind = CheckpointKind.Checkpoint;
		}
		draw();
	}

	@Override
	protected CheckpointBlock subClone() {
		return new CheckpointBlock(getX(), getY());
	}

	@Override
	public String toString() {
		return "Checkpoint Block [" + kind + "] at (" + getX() + "|" + getY()
				+ ")";
	}

	@Override
	public String getTypeString() {
		return "Checkpoint";
	}

	@Override
	protected Node getSaturationMenu() {
		TitledPane saturationMenu = new TitledPane();
		saturationMenu.setCollapsible(false);
		saturationMenu.setText("Checkpoint type");

		final ListSpinner<Integer> pointsSpinner = new ListSpinner<>();

		GridPane controls = new GridPane();
		controls.getStyleClass().add("grid-pane");

		final ComboBox<CheckpointKind> kindCombo = new ComboBox<>(
				FXCollections.observableArrayList(CheckpointKind.values()));
		kindCombo.getSelectionModel().select(kind);
		kindCombo.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(final ActionEvent pEvent) {
				kind = kindCombo.getValue();
				setupPointSpinner(pointsSpinner);
				changed(null, null, null);
			}
		});
		controls.add(kindCombo, 0, 0);

		Label pointsLabel = new Label("Points:");
		pointsLabel.disableProperty().bind(
				kindCombo.valueProperty().isEqualTo(CheckpointKind.Start));
		pointsLabel.visibleProperty().bind(
				kindCombo.valueProperty().isNotEqualTo(CheckpointKind.Start));
		controls.add(pointsLabel, 0, 1);

		ListSpinnerCaspianSkin<Integer> pointsSpinnerSkin = new ListSpinnerCaspianSkin<>(
				pointsSpinner);
		pointsSpinnerSkin.setArrowDirection(ArrowDirection.HORIZONTAL);
		pointsSpinnerSkin.setArrowPosition(ArrowPosition.SPLIT);
		pointsSpinnerSkin.setMaxWidth(100);
		pointsSpinner.setSkin(pointsSpinnerSkin);
		pointsSpinner.setEditable(false);
		pointsSpinner.disableProperty().bind(
				kindCombo.valueProperty().isEqualTo(CheckpointKind.Start));
		pointsSpinner.visibleProperty().bind(
				kindCombo.valueProperty().isNotEqualTo(CheckpointKind.Start));
		setupPointSpinner(pointsSpinner);
		controls.add(pointsSpinner, 1, 1);

		final CheckBox mandatoryCheckBox = new CheckBox("Mandatory Checkpoint");
		mandatoryCheckBox.setSelected(mandatory.get());
		mandatoryCheckBox.disableProperty().bind(
				kindCombo.valueProperty().isNotEqualTo(
						CheckpointKind.Checkpoint));
		mandatoryCheckBox.visibleProperty().bind(
				kindCombo.valueProperty().isEqualTo(CheckpointKind.Checkpoint));
		mandatory.bind(mandatoryCheckBox.selectedProperty());
		controls.add(mandatoryCheckBox, 0, 2);
		GridPane.setColumnSpan(mandatoryCheckBox, 2);

		saturationMenu.setContent(controls);

		return saturationMenu;
	}

	private void setupPointSpinner(final ListSpinner<Integer> pointsSpinner) {
		if (kind == CheckpointKind.Start) {
			return;
		}
		points.unbind();
		if (kind == CheckpointKind.Checkpoint) {
			pointsSpinner.setItems(checkpointPointOffsets
					.getObservableValueList());
			if (checkpointPointOffsets.getByValue(points.get()) != null) {
				pointsSpinner.setValue(points.get());
			} else {
				pointsSpinner.setValue(checkpointPointOffsets.getMinValue());
			}
		} else {
			pointsSpinner.setItems(endPointOffsets.getObservableValueList());
			if (endPointOffsets.getByValue(points.get()) != null) {
				pointsSpinner.setValue(points.get());
			} else {
				pointsSpinner.setValue(endPointOffsets.getMinValue());
			}
		}
		points.bind(pointsSpinner.valueProperty());
	}
}
