package level.blocks;

import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleDoubleProperty;
import javafx.collections.FXCollections;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.scene.Node;
import javafx.scene.control.CheckBox;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.control.TitledPane;
import javafx.scene.layout.GridPane;

/**
 * Appear or disappear periodically or button-triggered.
 */
public class GhostBlock extends StandardBlock {

	/**
	 * The initial state of the Block.
	 */
	public enum InitialState {
		/** On level start, the block will be visible. */
		present,

		/** On level start, the block will be invisible. */
		hidden
	}

	/** The initial state of the block. */
	private InitialState initialState;

	private static OffsetValueList<Double> durationOffsets = new OffsetValueList<>(
			new OffsetValue<>(0, 0.5), new OffsetValue<>(3, 1.0),
			new OffsetValue<>(6, 1.5), new OffsetValue<>(9, 2.0),
			new OffsetValue<>(12, 2.5), new OffsetValue<>(15, 3.0),
			new OffsetValue<>(20, Double.POSITIVE_INFINITY));

	private SimpleBooleanProperty autoToggle;

	private SimpleDoubleProperty duration;

	/**
	 * Instantiates a new Ghost block.
	 * 
	 * @param pX
	 *            the x coordinate
	 * @param pY
	 *            the y coordinate
	 */
	public GhostBlock(final int pX, final int pY) {
		super(pX, pY);
	}

	@Override
	protected void init() {
		super.init();
		initialState = InitialState.present;
		duration = new SimpleDoubleProperty(1.5);
		duration.addListener(this);
		autoToggle = new SimpleBooleanProperty(true);
		autoToggle.addListener(this);
	}

	@Override
	protected int getHue() {
		return 225;
	}

	@Override
	protected double getSaturation() {
		int value = 0;
		switch (initialState) {
		case present:
			value = 80;
			break;
		case hidden:
			value = 50;
			break;
		}

		if (!autoToggle.get()) {
			return (value + 20) / 100.0;
		}
		return (value + durationOffsets.getByValue(duration.get()).getOffset()) / 100.0;
	}

	@Override
	public void setSaturation(final double pSaturation) {
		int saturation = (int) (100 * pSaturation + 0.5);
		OffsetValue<Double> d = null;

		if (saturation >= 79) { // ">= 80"
			initialState = InitialState.present;
			d = durationOffsets.getByOffset(saturation - 80);
		} else { // ">= 50" and fallback
			initialState = InitialState.hidden;
			d = durationOffsets.getByOffset(saturation - 50);
		}

		if (d == null || Double.isInfinite(d.getValue())) {
			autoToggle.set(false);
			duration.set(1.5);
		} else {
			autoToggle.set(true);
			duration.set(d.getValue());
		}
		draw();
	}

	@Override
	protected boolean needsId() {
		return true;
	}

	@Override
	protected GhostBlock subClone() {
		return new GhostBlock(getX(), getY());
	}

	@Override
	public String toString() {
		return "Ghost Block [" + duration.get() + "] at (" + getX() + "|"
				+ getY() + ")";
	}

	@Override
	public String getTypeString() {
		return "Ghost";
	}

	@Override
	protected Node getSaturationMenu() {
		TitledPane saturationMenu = new TitledPane();
		saturationMenu.setCollapsible(false);
		saturationMenu.setText("Ghost properties");

		GridPane controls = new GridPane();
		controls.getStyleClass().add("grid-pane");

		controls.add(new Label("Initial State:"), 0, 0);
		final ComboBox<InitialState> initialCombo = new ComboBox<>(
				FXCollections.observableArrayList(InitialState.values()));
		initialCombo.getSelectionModel().select(initialState);
		initialCombo.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(final ActionEvent pEvent) {
				initialState = initialCombo.getValue();
				changed(null, null, null);
			}
		});
		controls.add(initialCombo, 1, 0);

		final CheckBox autoToggleCheckBox = new CheckBox(
				"Toggle State automatically");
		autoToggleCheckBox.setSelected(autoToggle.get());
		autoToggle.bind(autoToggleCheckBox.selectedProperty());
		controls.add(autoToggleCheckBox, 0, 1);
		GridPane.setColumnSpan(autoToggleCheckBox, 2);

		Label durationLabel = new Label("Duration:");
		durationLabel.disableProperty().bind(autoToggle.not());
		controls.add(durationLabel, 0, 2);

		ComboBox<Double> durationCombo = new ComboBox<>(
				FXCollections.observableArrayList(0.5, 1.0, 1.5, 2.0, 2.5, 3.0));
		durationCombo.getSelectionModel().select(duration.get());
		durationCombo.disableProperty().bind(autoToggle.not());
		duration.bind(durationCombo.valueProperty());
		controls.add(durationCombo, 1, 2);

		saturationMenu.setContent(controls);

		return saturationMenu;
	}
}
