package level.blocks;

import javafx.beans.property.SimpleIntegerProperty;
import javafx.collections.FXCollections;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.scene.Node;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.control.TitledPane;
import javafx.scene.layout.GridPane;
import javafx.util.StringConverter;
import jfxtras.labs.internal.scene.control.skin.ListSpinnerCaspianSkin;
import jfxtras.labs.internal.scene.control.skin.ListSpinnerCaspianSkin.ArrowDirection;
import jfxtras.labs.internal.scene.control.skin.ListSpinnerCaspianSkin.ArrowPosition;
import jfxtras.labs.scene.control.ListSpinner;

/**
 * Lose or gain points, lives, become weightless...
 */
public class ItemBlock extends StandardBlock {

	/**
	 * The different types of Items.
	 */
	public enum ItemKind {
		/** The player receives extra points. */
		GoldBonus("Amount", "Little", "Moderate", "A whole lot"),

		/** The player loses points. */
		GoldPenalty("Amount", "Little", "Moderate", "A whole lot"),

		/** The player receives lives. */
		LifeBonus("Amount", "One", "Two", "Three"),

		/** The player loses lives. */
		LifePenalty("Amount", "One", "Two", "Three"),

		/** The ball ignores gravity for a while. */
		AntiGravity("Duration", "Short", "Medium", "Long");

		/** The parameter description. */
		public final String parameter;

		/** The low label. */
		public final String low;

		/** The mid label. */
		public final String mid;

		/** The high label. */
		public final String high;

		/**
		 * Instantiates a new trap kind.
		 * 
		 * @param pParameter
		 *            the parameter description
		 * @param pLow
		 *            the low label
		 * @param pMid
		 *            the mid label
		 * @param pHigh
		 *            the high label
		 */
		ItemKind(final String pParameter, final String pLow, final String pMid,
				final String pHigh) {
			parameter = pParameter;
			low = pLow;
			mid = pMid;
			high = pHigh;
		}
	}

	/** The kind of the trap. */
	private ItemKind kind;

	/** The parameter offset. */
	private SimpleIntegerProperty parameter;

	private static OffsetValueList<Integer> parameters = new OffsetValueList<>(
			new OffsetValue<>(0, 0), new OffsetValue<>(3, 3),
			new OffsetValue<>(6, 6));

	/**
	 * Instantiates a new Trap block.
	 * 
	 * @param pX
	 *            the x coordinate
	 * @param pY
	 *            the y coordinate
	 */
	public ItemBlock(final int pX, final int pY) {
		super(pX, pY);
	}

	@Override
	protected void init() {
		super.init();
		kind = ItemKind.GoldBonus;
		parameter = new SimpleIntegerProperty(0);
		parameter.addListener(this);
	}

	@Override
	protected int getHue() {
		return 45;
	}

	@Override
	protected double getSaturation() {
		int value = 90 - kind.ordinal() * 10;

		return (value + parameter.get()) / 100.0;
	}

	@Override
	public void setSaturation(final double pSaturation) {
		int saturation = (int) (pSaturation * 100.0 + 0.5);
		int offset = saturation;
		if (saturation >= 89) { // "== 90"
			kind = ItemKind.GoldBonus;
			offset -= 90;
		} else if (saturation >= 79) { // "== 80"
			kind = ItemKind.GoldPenalty;
			offset -= 80;
		} else if (saturation >= 69) { // "== 70"
			kind = ItemKind.LifeBonus;
			offset -= 70;
		} else if (saturation >= 59) { // "== 60"
			kind = ItemKind.LifePenalty;
			offset -= 60;
		} else if (saturation >= 49) { // "== 50"
			kind = ItemKind.AntiGravity;
			offset -= 50;
		} else { // fallback
			kind = ItemKind.GoldBonus;
			offset = 0;
		}
		OffsetValue<Integer> i = parameters.getByOffset(offset);
		if (i != null) {
			parameter.set(i.getValue());
		} else {
			parameter.set(0);
		}
		draw();
	}

	@Override
	protected boolean needsId() {
		return false;
	}

	@Override
	protected ItemBlock subClone() {
		return new ItemBlock(getX(), getY());
	}

	@Override
	public String toString() {
		return "Item Block [" + kind + "] at (" + getX() + "|" + getY() + ")";
	}

	@Override
	public String getTypeString() {
		return "Item";
	}

	@Override
	protected Node getSaturationMenu() {
		TitledPane saturationMenu = new TitledPane();
		saturationMenu.setCollapsible(false);
		saturationMenu.setText("Item type");

		final Label parameterLabel = new Label(kind.parameter + ":");
		final ListSpinner<Integer> parameterSpinner = new ListSpinner<>(0, 6, 3);

		GridPane controls = new GridPane();
		controls.getStyleClass().add("grid-pane");

		final ComboBox<ItemKind> kindCombo = new ComboBox<>(
				FXCollections.observableArrayList(ItemKind.values()));
		kindCombo.getSelectionModel().select(kind);
		kindCombo.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(final ActionEvent pEvent) {
				kind = kindCombo.getValue();
				parameterLabel.setText(kind.parameter + ":");
				int p = parameter.get();
				parameterSpinner.increment();
				parameterSpinner.decrement();
				parameterSpinner.setValue(p);
				changed(null, null, null);
			}
		});
		controls.add(kindCombo, 0, 0);

		controls.add(parameterLabel, 0, 1);

		ListSpinnerCaspianSkin<Integer> pointsSpinnerSkin = new ListSpinnerCaspianSkin<>(
				parameterSpinner);
		pointsSpinnerSkin.setArrowDirection(ArrowDirection.HORIZONTAL);
		pointsSpinnerSkin.setArrowPosition(ArrowPosition.SPLIT);
		pointsSpinnerSkin.setMaxWidth(100);
		parameterSpinner.setSkin(pointsSpinnerSkin);
		parameterSpinner.setEditable(false);

		parameterSpinner.setStringConverter(new StringConverter<Integer>() {
			@Override
			public String toString(final Integer value) {
				switch (value) {
				case 0:
					return kind.low;
				case 3:
					return kind.mid;
				case 6:
					return kind.high;
				default:
					return "";
				}
			}

			@Override
			public Integer fromString(final String label) {
				if (label.equals(kind.low)) {
					return 0;
				} else if (label.equals(kind.mid)) {
					return 3;
				} else if (label.equals(kind.high)) {
					return 6;
				} else {
					return 0;
				}
			}
		});

		parameterSpinner.increment();
		parameterSpinner.setValue(parameter.get());
		parameter.bind(parameterSpinner.valueProperty());
		controls.add(parameterSpinner, 1, 1);

		saturationMenu.setContent(controls);

		return saturationMenu;
	}
}
