package level.blocks;

import java.util.ArrayList;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;

/**
 * A List of OffsetValues that can be asked for an OffsetValue by its offset or
 * value.
 * 
 * @param <T>
 *            the value type
 */
@SuppressWarnings("serial")
public class OffsetValueList<T extends Comparable<T>> extends
		ArrayList<OffsetValue<T>> {

	/**
	 * Instantiates a new duration list.
	 * 
	 * @param offsetValues
	 *            the offsetValues.
	 */
	@SafeVarargs
	public OffsetValueList(final OffsetValue<T>... offsetValues) {
		super();
		for (OffsetValue<T> d : offsetValues) {
			add(d);
		}
	}

	/**
	 * Gets the OffsetValue by its offset.
	 * 
	 * @param offset
	 *            the offset
	 * @return the corresponding OffsetValue
	 */
	public OffsetValue<T> getByOffset(final int offset) {
		for (OffsetValue<T> d : this) {
			if (d.isOffset(offset)) {
				return d;
			}
		}
		return null;
	}

	/**
	 * Gets the OffsetValue by its value.
	 * 
	 * @param value
	 *            the value
	 * @return the corresponding OffsetValue
	 */
	public OffsetValue<T> getByValue(final T value) {
		for (OffsetValue<T> d : this) {
			if (d.isValue(value)) {
				return d;
			}
		}
		return null;
	}

	/**
	 * Gets the minimum value.
	 * 
	 * @return the minimum value
	 */
	public T getMinValue() {
		T min = null;
		for (OffsetValue<T> d : this) {
			if (min == null || d.getValue().compareTo(min) < 0) {
				min = d.getValue();
			}
		}
		return min;
	}

	/**
	 * Gets the maximum value.
	 * 
	 * @return the maximum value
	 */
	public T getMaxValue() {
		T max = null;
		for (OffsetValue<T> d : this) {
			if (max == null || d.getValue().compareTo(max) > 0) {
				max = d.getValue();
			}
		}
		return max;
	}

	/**
	 * Gets an observable list of all values.
	 * 
	 * @return the observable value list
	 */
	public ObservableList<T> getObservableValueList() {
		ObservableList<T> list = FXCollections.observableArrayList();
		for (OffsetValue<T> d : this) {
			list.add(d.getValue());
		}
		FXCollections.sort(list);
		return list;
	}
}