package level.blocks;

import javafx.beans.property.SimpleIntegerProperty;
import javafx.collections.FXCollections;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.scene.Node;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.control.TitledPane;
import javafx.scene.layout.GridPane;
import javafx.util.StringConverter;
import jfxtras.labs.internal.scene.control.skin.ListSpinnerCaspianSkin;
import jfxtras.labs.internal.scene.control.skin.ListSpinnerCaspianSkin.ArrowDirection;
import jfxtras.labs.internal.scene.control.skin.ListSpinnerCaspianSkin.ArrowPosition;
import jfxtras.labs.scene.control.ListSpinner;

/**
 * Fall through a wormhole to a target Portal with the same ID
 */
public class PortalBlock extends StandardBlock {

	/**
	 * The different orientations of Portals.
	 */
	public enum PortalOrientation {
		/** The Portal is in +x direction. */
		Right,

		/** The Portal is in -x direction. */
		Left,

		/** The Portal is in +y direction. */
		Back,

		/** The Portal is in -y direction. */
		Front,

		/** The Portal is in +z direction. */
		Top;
	}

	/** The orientation of the portal. */
	private PortalOrientation orientation;

	/** The parameter offset. */
	private SimpleIntegerProperty parameter;

	private static OffsetValueList<Integer> parameters = new OffsetValueList<>(
			new OffsetValue<>(0, 0), new OffsetValue<>(3, 3),
			new OffsetValue<>(6, 6));

	/**
	 * Instantiates a new Portal block.
	 * 
	 * @param pX
	 *            the x coordinate
	 * @param pY
	 *            the y coordinate
	 */
	public PortalBlock(final int pX, final int pY) {
		super(pX, pY);
	}

	@Override
	protected void init() {
		super.init();
		orientation = PortalOrientation.Right;
		parameter = new SimpleIntegerProperty(0);
		parameter.addListener(this);
	}

	@Override
	protected int getHue() {
		return 315;
	}

	@Override
	protected double getSaturation() {
		int value = 90 - orientation.ordinal() * 10;

		return (value + parameter.get()) / 100.0;
	}

	@Override
	public void setSaturation(final double pSaturation) {
		int saturation = (int) (pSaturation * 100.0 + 0.5);
		int offset = saturation;
		if (saturation >= 89) { // "== 90"
			orientation = PortalOrientation.Right;
			offset -= 90;
		} else if (saturation >= 79) { // "== 80"
			orientation = PortalOrientation.Left;
			offset -= 80;
		} else if (saturation >= 69) { // "== 70"
			orientation = PortalOrientation.Back;
			offset -= 70;
		} else if (saturation >= 59) { // "== 60"
			orientation = PortalOrientation.Front;
			offset -= 60;
		} else if (saturation >= 49) { // "== 50"
			orientation = PortalOrientation.Top;
			offset -= 50;
		} else { // fallback
			orientation = PortalOrientation.Right;
			offset = 0;
		}
		OffsetValue<Integer> i = parameters.getByOffset(offset);
		if (i != null) {
			parameter.set(i.getValue());
		} else {
			parameter.set(0);
		}
		draw();
	}

	@Override
	protected boolean needsId() {
		return true;
	}

	@Override
	protected PortalBlock subClone() {
		return new PortalBlock(getX(), getY());
	}

	@Override
	public String toString() {
		return "Portal Block [" + orientation + "] at (" + getX() + "|"
				+ getY() + ")";
	}

	@Override
	public String getTypeString() {
		return "Portal";
	}

	@Override
	protected Node getSaturationMenu() {
		TitledPane saturationMenu = new TitledPane();
		saturationMenu.setCollapsible(false);
		saturationMenu.setText("Portal type");

		GridPane controls = new GridPane();
		controls.getStyleClass().add("grid-pane");
		controls.add(new Label("Orientation:"), 0, 0);
		final ComboBox<PortalOrientation> kindCombo = new ComboBox<>(
				FXCollections.observableArrayList(PortalOrientation.values()));
		kindCombo.getSelectionModel().select(orientation);
		kindCombo.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(final ActionEvent pEvent) {
				orientation = kindCombo.getValue();
				changed(null, null, null);
			}
		});
		controls.add(kindCombo, 1, 0);

		controls.add(new Label("Attraction:"), 0, 1);
		final ListSpinner<Integer> parameterSpinner = new ListSpinner<>(0, 6, 3);
		ListSpinnerCaspianSkin<Integer> pointsSpinnerSkin = new ListSpinnerCaspianSkin<>(
				parameterSpinner);
		pointsSpinnerSkin.setArrowDirection(ArrowDirection.HORIZONTAL);
		pointsSpinnerSkin.setArrowPosition(ArrowPosition.SPLIT);
		pointsSpinnerSkin.setMaxWidth(100);
		parameterSpinner.setSkin(pointsSpinnerSkin);
		parameterSpinner.setEditable(false);

		parameterSpinner.setStringConverter(new StringConverter<Integer>() {
			@Override
			public String toString(final Integer value) {
				switch (value) {
				case 0:
					return "deactivated";
				case 3:
					return "low";
				case 6:
					return "high";
				default:
					return "";
				}
			}

			@Override
			public Integer fromString(final String label) {
				if (label.equals("deactivated")) {
					return 0;
				} else if (label.equals("low")) {
					return 3;
				} else if (label.equals("high")) {
					return 6;
				} else {
					return 0;
				}
			}
		});
		parameterSpinner.increment();
		parameterSpinner.setValue(parameter.get());
		parameter.bind(parameterSpinner.valueProperty());
		controls.add(parameterSpinner, 1, 1);

		saturationMenu.setContent(controls);

		return saturationMenu;
	}
}
