package level.blocks;

import javafx.beans.property.SimpleIntegerProperty;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.scene.Node;
import javafx.scene.control.ComboBox;
import javafx.scene.control.Label;
import javafx.scene.control.TitledPane;
import javafx.scene.layout.GridPane;
import javafx.util.converter.IntegerStringConverter;
import jfxtras.labs.internal.scene.control.skin.ListSpinnerCaspianSkin;
import jfxtras.labs.internal.scene.control.skin.ListSpinnerCaspianSkin.ArrowDirection;
import jfxtras.labs.scene.control.ListSpinner;

/**
 * A block without any abilities for a simple wall or ground.
 */
public class StandardBlock extends Block {

	/**
	 * Instantiates a new standard block.
	 * 
	 * @param pX
	 *            the x coordinate
	 * @param pY
	 *            the y coordinate
	 */
	public StandardBlock(final int pX, final int pY) {
		super(pX, pY);
	}

	@Override
	protected void init() {
		super.init();
		startHeight = new SimpleIntegerProperty(2);
		startHeight.addListener(this);
		endHeight = new SimpleIntegerProperty(2);
		endHeight.addListener(this);
		groupId = new SimpleIntegerProperty(255);
		gradientDirection = GradientDirection.top;
	}

	@Override
	public final boolean isHue(final int pHue) {
		return (pHue >= getHue() - 10 && pHue <= getHue() + 10)
				|| (pHue - 360 >= getHue() - 10 && pHue - 360 <= getHue() + 10);
	}

	@Override
	protected int getHue() {
		return 90;
	}

	@Override
	protected double getSaturation() {
		return 1.0;
	}

	@Override
	public void setSaturation(final double pSaturation) {
		return;
	}

	/** The start height of the gradient. */
	private SimpleIntegerProperty startHeight;

	/** The end height of the gradient. */
	private SimpleIntegerProperty endHeight;

	/** The gradient direction. */
	private GradientDirection gradientDirection;

	@Override
	protected final double getBrightness() {
		if (startHeight.get() == 0 && endHeight.get() == 0) {
			return 0;
		}

		int brightness = 0; // start value

		switch (startHeight.get()) {
		case 0:
			brightness = 56;
			break;
		case 1:
			brightness = 73;
			break;
		case 2:
			brightness = 86;
			break;
		case 3:
			brightness = 95;
			break;
		case 4:
			brightness = 100;
			break;
		}

		if (startHeight.get() != endHeight.get()) {
			brightness++;
			brightness += (endHeight.get() - startHeight.get() - 1)
					* GradientDirection.values().length;
			brightness += gradientDirection.ordinal();
		}

		return brightness / 100.0;
	}

	@Override
	public final void setBrightness(final double pBrightness) {
		startHeight.set(0);
		endHeight.set(0);
		gradientDirection = GradientDirection.top;

		int brightness = (int) (pBrightness * 100 + 0.5);
		int b = -1;

		if (brightness < 57) {
			startHeight.set(0);
			endHeight.set(0);
		} else if (brightness < 73) {
			startHeight.set(0);
			b = brightness - 57;
		} else if (brightness == 73) {
			startHeight.set(1);
			endHeight.set(1);
		} else if (brightness < 86) {
			startHeight.set(1);
			b = brightness - 74;
		} else if (brightness == 86) {
			startHeight.set(2);
			endHeight.set(2);
		} else if (brightness < 95) {
			startHeight.set(2);
			b = brightness - 87;
		} else if (brightness == 95) {
			startHeight.set(3);
			endHeight.set(3);
		} else if (brightness < 100) {
			startHeight.set(3);
			b = brightness - 96;
		} else if (brightness == 100) {
			startHeight.set(4);
			endHeight.set(4);
		}

		if (b != -1) {
			endHeight.set(b / 4 + startHeight.get() + 1);
			gradientDirection = GradientDirection.values()[b % 4];
		}

		draw();
	}

	/** The group id. */
	private SimpleIntegerProperty groupId;

	@Override
	protected final double getTransparency() {
		if (needsId()) {
			return groupId.doubleValue() / 255.0;
		}
		return 1.0;
	}

	@Override
	public final void setTransparency(final double pTransparency) {
		if (needsId()) {
			groupId.set((int) (pTransparency * 255.0 + 0.5));
		}
		draw();
	}

	@Override
	protected StandardBlock subClone() {
		return new StandardBlock(getX(), getY());
	}

	@Override
	public String toString() {
		return "Standard Block at (" + getX() + "|" + getY() + ")";
	}

	@Override
	public String getTypeString() {
		return "Standard";
	}

	private ComboBox<Integer> startHeightCombo, endHeightCombo;

	@Override
	protected final Node getBrightnessMenu() {
		TitledPane brightnessMenu = new TitledPane();
		brightnessMenu.setCollapsible(false);
		brightnessMenu.setText("Block shape");

		GridPane controls = new GridPane();
		controls.getStyleClass().add("grid-pane");

		controls.add(new Label("Start Height:"), 0, 0);
		startHeightCombo = new ComboBox<>(FXCollections.observableArrayList(0,
				1, 2, 3, 4));
		startHeightCombo.getSelectionModel().select(startHeight.get());
		startHeight.bind(startHeightCombo.valueProperty());
		controls.add(startHeightCombo, 1, 0);

		controls.add(new Label("End Height:"), 0, 1);
		endHeightCombo = new ComboBox<>(FXCollections.observableArrayList(0, 1,
				2, 3, 4));
		endHeightCombo.getSelectionModel().select(endHeight.get());
		endHeight.bind(endHeightCombo.valueProperty());
		controls.add(endHeightCombo, 1, 1);

		controls.add(new Label("Direction:"), 0, 2);
		final ComboBox<GradientDirection> directionCombo = new ComboBox<>(
				FXCollections.observableArrayList(GradientDirection.values()));
		directionCombo.getSelectionModel().select(gradientDirection);
		directionCombo.disableProperty().bind(
				startHeightCombo.valueProperty().isEqualTo(
						endHeightCombo.valueProperty()));
		directionCombo.setOnAction(new EventHandler<ActionEvent>() {
			@Override
			public void handle(final ActionEvent pEvent) {
				gradientDirection = directionCombo.getValue();
				changed(null, null, null);
			}
		});
		controls.add(directionCombo, 1, 2);

		brightnessMenu.setContent(controls);

		return brightnessMenu;
	}

	/**
	 * Specifies if this Block can make use of a group id.
	 * 
	 * @return true, if a id should be possible to specify
	 */
	protected boolean needsId() {
		return false;
	}

	@Override
	protected Node getTransparencyMenu() {
		if (!needsId()) {
			return null;
		}
		TitledPane transparencyMenu = new TitledPane();
		transparencyMenu.setCollapsible(false);
		transparencyMenu.setText("Group id");

		GridPane controls = new GridPane();
		controls.getStyleClass().add("grid-pane");

		controls.add(new Label("Group Id:"), 0, 0);
		ListSpinner<Integer> idSelector = new ListSpinner<>(0, 255);
		ListSpinnerCaspianSkin<Integer> skin = new ListSpinnerCaspianSkin<>(
				idSelector);
		skin.setArrowDirection(ArrowDirection.VERTICAL);
		idSelector.setSkin(skin);
		idSelector.setEditable(true);
		idSelector.setStringConverter(new IntegerStringConverter());
		idSelector.setValue(groupId.get());
		groupId.bind(idSelector.valueProperty());
		idSelector.valueProperty().addListener(this);
		controls.add(idSelector, 1, 0);

		transparencyMenu.setContent(controls);

		return transparencyMenu;
	}

	@Override
	public void changed(final ObservableValue<? extends Object> pObservable,
			final Object pOldValue, final Object pNewValue) {
		if (startHeightCombo != null & endHeightCombo != null) {
			if (startHeightCombo.getValue() > endHeightCombo.getValue()) {
				int start = Math.min(startHeightCombo.getValue(),
						endHeightCombo.getValue());
				int end = Math.max(startHeightCombo.getValue(),
						endHeightCombo.getValue());
				startHeightCombo.getSelectionModel().select(start);
				endHeightCombo.getSelectionModel().select(end);
			}
		}
		super.changed(pObservable, pOldValue, pNewValue);
	}
}
